{-# LANGUAGE TemplateHaskell #-}

module HAL.Doc.Corpus where

import GHC.Generics

import Control.Applicative ((<|>))
import qualified Control.Lens as L
import Data.Aeson
import Data.Aeson.Types (Parser)
import Data.Default
import Data.Text (pack, unpack, Text)
import Text.Read (readMaybe)

import Servant.API (ToHttpApiData(..))

data Corpus = Corpus
  { _corpus_docid         :: Text
  , _corpus_title         :: [Text]
  , _corpus_abstract      :: [Text]
  , _corpus_date          :: Maybe Text
  , _corpus_source        :: Maybe Text
  , _corpus_authors_names :: [Text]
  , _corpus_authors_affiliations :: [Text]
  , _corpus_struct_id     :: [Int]
  } deriving (Show, Generic)
L.makeLenses ''Corpus

instance Default Corpus where
  def = Corpus "default Id" def def def def def def def

instance FromJSON Corpus where
  parseJSON (Object o) =
    Corpus
      <$> o .: "docid"
      <*> (o .: "title_s" <|> return [])
      <*> (o .: "abstract_s" <|> return [])
      <*> (o .:? "submittedDate_s")
      <*> (o .:? "source_s")
      <*> (o .: "authFullName_s" <|> return [])
      <*> (o .: "authOrganism_s" <|> return [])
      <*> (o .: "structId_i" <|> return [])

instance ToHttpApiData Corpus where
  toUrlPiece _ = "docid,title_s,abstract_s,submittedDate_s,source_s,authFullName_s,authOrganism_s,structId_i"
